/******************************************************************************/
/* Agent2b - Vending Machine                                                  */
/******************************************************************************/
/* Nome: 8661_VMNSTP_GERA_QTAVISTA                                            */
/*                                                                            */
/* Descrio:                                                                 */
/*  Gera a quantidade vendida a vista de acordo com o valor da sangria        */
/*                                                                            */
/* Criao:                                                                   */
/* 15/07/2003 - Andr Rodrigo                                                 */
/******************************************************************************/
/* raise_application_error(-20501, '' , True);                                */
/* @C:\Win_V2\Sql\End_Conv\Oracle\8661_VMNSTP_GERA_QTAVISTA.SQL               */
/******************************************************************************/

/******************************************************************************/
/* GRAVA O LANAMENTO DE ESTOQUE PARA OS ITENS DO MAPA                        */
/******************************************************************************/
CREATE OR REPLACE PROCEDURE GERA_QTAVISTA
   (P_CDFILIAL     IN  MOVIVALOR.CDFILIAL%TYPE,
    P_DTMOVIVALOR  IN  MOVIVALOR.DTMOVIVALOR%TYPE,
    P_IDERRO       OUT MOVIVALOR.IDTIPMOVIVLR%TYPE)
IS
   V_CTSANGRIA      NUMBER;
   V_VRTOTAL        NUMBER;
   V_VRPESANG       NUMBER;
   V_VRTOTDIF       NUMBER;
   V_VRAVISTA       NUMBER;
   V_QTAVISTA       NUMBER;
   V_QTTEMP         NUMBER;
   V_VRDESC         NUMBER;
   V_VRTOTALPROD    NUMBER;
   V_VRUNITPROD     NUMBER;

   V_CDEQUIPPRO     MAPABASEV.CDEQUIPPRO%TYPE;
   V_CDROTATRANS    MAPABASEV.CDROTATRANS%TYPE;

   V_VRMOVIVALOR    MOVIVALOR.VRMOVIVALOR%TYPE;
   V_IDTPPGTOEQUI   EQUIPAPROD.IDTPPGTOEQUI%TYPE;

   V_CDFILIAL       ITVENDAVMN.CDFILIAL%TYPE;
   V_NRVENDA        ITVENDAVMN.NRVENDA%TYPE;
   V_NRSEQITVENDA   ITVENDAVMN.NRSEQITVENDA%TYPE;
BEGIN
   /* P_IDERRO
        A = OK
        B = SEM SANGRIA VLIDA
        C = VALOR A PRAZO IGUAL A ZERO
   */
   SELECT COUNT(*) INTO V_CTSANGRIA
   FROM MOVIVALOR
   WHERE CDFILIAL     = P_CDFILIAL
     AND DTMOVIVALOR  = P_DTMOVIVALOR
     AND IDTIPMOVIVLR = '2';

   P_IDERRO := 'A';
   IF V_CTSANGRIA = 0 THEN
      P_IDERRO := 'B';
   ELSE
      /* RETORNA OS DADOS DO MAPA RELACIONA DA SANGRIA */
      SELECT
         MAX(MA.CDEQUIPPRO), MAX(MA.CDROTATRANS), MAX(VRMOVIVALOR)
      INTO
         V_CDEQUIPPRO, V_CDROTATRANS, V_VRMOVIVALOR
      FROM
         MOVIVALOR MV, MAPABASEV MA
      WHERE (MV.CDFILIAL     = P_CDFILIAL)
        AND (MV.DTMOVIVALOR  = P_DTMOVIVALOR)
        AND (MV.IDTIPMOVIVLR = '2')
        AND (MV.CDFILIAL     = MA.CDFILIAL)
        AND (MV.NRMAPABASEV  = MA.NRMAPABASEV);

      /* TESTA SE O TIPO DE PAGAMENTO DA MQUINA  A VISTA E A PRAZO */
      SELECT MAX(IDTPPGTOEQUI) INTO V_IDTPPGTOEQUI
      FROM EQUIPAPROD
      WHERE CDEQUIPPRO = V_CDEQUIPPRO;

      IF V_IDTPPGTOEQUI = '3' THEN
         SELECT SUM(IT.QTFATUVENDA * (IT.VRUNITVENDA + IT.VRUNITVENDAC))
         INTO   V_VRTOTAL
         FROM   VENDAVMN VE, ITVENDAVMN IT, MAPABASEV MP
         WHERE (MP.CDFILIAL      = P_CDFILIAL     )
           AND (MP.DTMOVIVALOR   = P_DTMOVIVALOR  )
           AND (VE.IDSITVENDA    = '1'            )
           AND (VE.CDFILIAL      = MP.CDFILIAL    )
           AND (VE.NRMAPABASEV   = MP.NRMAPABASEV )
           AND (VE.CDFILIAL      = IT.CDFILIAL    )
           AND (VE.NRVENDA       = IT.NRVENDA     );

         IF V_VRTOTAL > 0 THEN
            V_VRPESANG := NVL(V_VRMOVIVALOR, 0) / V_VRTOTAL;
            V_VRTOTDIF := 0;
            FOR C_ITVENDA IN
              (SELECT
                  IT.CDFILIAL,  IT.NRVENDA, IT.NRSEQITVENDA,
                  IT.CDPRODUTO, IT.QTFATUVENDA,
                  (IT.VRUNITVENDA + IT.VRUNITVENDAC) AS VRUNITPROD,
                  (IT.QTFATUVENDA * (IT.VRUNITVENDA + IT.VRUNITVENDAC)) AS VRTOTALPROD
               FROM   VENDAVMN VE, ITVENDAVMN IT, MAPABASEV MP
               WHERE (MP.CDFILIAL      = P_CDFILIAL     )
                 AND (MP.DTMOVIVALOR   = P_DTMOVIVALOR  )
                 AND (VE.IDSITVENDA    = '1'            )
                 AND (VE.CDFILIAL      = MP.CDFILIAL    )
                 AND (VE.NRMAPABASEV   = MP.NRMAPABASEV )
                 AND (VE.CDFILIAL      = IT.CDFILIAL    )
                 AND (VE.NRVENDA       = IT.NRVENDA     )
               ORDER BY VRTOTALPROD) LOOP

               V_VRAVISTA := C_ITVENDA.VRTOTALPROD * V_VRPESANG;
               V_QTAVISTA := TRUNC(C_ITVENDA.QTFATUVENDA * V_VRPESANG);

               V_VRTOTDIF := V_VRTOTDIF + (V_VRAVISTA - (V_QTAVISTA * C_ITVENDA.VRUNITPROD));

               UPDATE ITVENDAVMN
               SET
                  QTPRODVENDA  = V_QTAVISTA,
                  QTFATUVENDA  = QTFATUVENDA - V_QTAVISTA,
                  QTFATORIGVMN = C_ITVENDA.QTFATUVENDA
               WHERE CDFILIAL     = C_ITVENDA.CDFILIAL
                 AND NRVENDA      = C_ITVENDA.NRVENDA
                 AND NRSEQITVENDA = C_ITVENDA.NRSEQITVENDA;

               V_CDFILIAL     := C_ITVENDA.CDFILIAL;
               V_NRVENDA      := C_ITVENDA.NRVENDA;
               V_NRSEQITVENDA := C_ITVENDA.NRSEQITVENDA;
               V_VRTOTALPROD  := C_ITVENDA.VRTOTALPROD;
               V_VRUNITPROD   := C_ITVENDA.VRUNITPROD;
            END LOOP;

            /* GRAVA A DIFERENA NO ULTIMO PRODUTO */
            IF V_VRTOTDIF > 0 THEN
               V_QTTEMP   := V_VRTOTDIF / V_VRUNITPROD;
               V_QTAVISTA := TRUNC(V_VRTOTDIF / V_VRUNITPROD);

               IF V_QTAVISTA <> V_QTTEMP THEN
                  V_QTAVISTA := V_QTAVISTA + 1;
               END IF;

               V_VRDESC := V_VRTOTDIF - (V_QTAVISTA * V_VRUNITPROD);

               UPDATE ITVENDAVMN
               SET
                  QTPRODVENDA  = QTPRODVENDA + V_QTAVISTA,
                  QTFATUVENDA  = QTFATUVENDA - V_QTAVISTA,
                  VRDESCITVEND = V_VRDESC
               WHERE CDFILIAL     = V_CDFILIAL
                 AND NRVENDA      = V_NRVENDA
                 AND NRSEQITVENDA = V_NRSEQITVENDA;
            END IF;
         ELSE
            P_IDERRO := 'C';
         END IF;
      END IF;
   END IF;
END;
/

